﻿Option Explicit On
Option Strict Off
Imports System.Data
Imports System.Data.Sql
Imports System.Data.SqlClient
Imports System.IO
'Following for SmptClient
Imports System.Net
Imports System.Net.Mail

'TODO for enabling DbMail and SmptClient automated reorder messages
'1. Change the RefreshOrders procedure’s strToEmail value to the destination e-mail address.
'2. Replace the SendReordersBySMTP procedure’s strHost, strFromEmail, and strPassword  placeholders with the values required by your e-mail provider. 

Public Class frmNotifications
    Private strProductsFile As String = Application.StartupPath + "\ProductsCurrent.xml"
    Private strSuppliersFile As String = Application.StartupPath + "\SuppliersCurrent.xml"
    Private cnNwind As SqlConnection     'Connection for dependency command
    Private cmdProds As SqlCommand   'Dependency command
    Private strDepMsg As String = Nothing
    Private depProds As SqlDependency 'SqlDependency on the Products table
    Private cmdRequest As SqlCommand     'SqlNotificationRequest command
    Private snrProds As SqlNotificationRequest
    Private strSQL As String = Nothing

    Private Sub Form_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'TODO: This line of code loads data into the 'NorthwindDataSet.Suppliers' table. You can move, or remove it, as needed.
        Me.SuppliersTableAdapter.Fill(Me.NorthwindDataSet.Suppliers)
        'Load the DataGridView
        Try
            With NorthwindDataSet.Products
                If File.Exists(strProductsFile) Then
                    'Load the current copy
                    .ReadXml(strProductsFile)
                Else
                    ProductsTableAdapter.Fill(NorthwindDataSet.Products)
                    'Create the current copy
                    .WriteXml(strProductsFile, XmlWriteMode.DiffGram)
                End If
                'Debugging
                Dim intRows As Integer = .Rows.Count
            End With
            With NorthwindDataSet.Suppliers
                If File.Exists(strSuppliersFile) Then
                    'Load the current copy
                    .ReadXml(strSuppliersFile)
                Else
                    SuppliersTableAdapter.Fill(NorthwindDataSet.Suppliers)
                    'Create the current copy
                    .WriteXml(strSuppliersFile, XmlWriteMode.DiffGram)
                End If
                'Debugging
                Dim intRows As Integer = .Rows.Count
            End With
        Catch exc As Exception
            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "DataSet Fill Operation Failed")
        End Try
        Application.DoEvents()
        'Default query
        strSQL = "SELECT UnitsInStock, UnitsOnOrder, ReorderLevel FROM dbo.Products"
        btnSaveChanges.Enabled = False
        txtHelp.Text = "Default query: " + strSQL
        Try
            'Check for pending SqlNotificationRequest messages
            Dim intPollResult As Integer = PollNotifications(True)
            If intPollResult = 1 Then
                'There's a notification pending
                If Not ListSubscriptions(True) Then
                    'Don't add duplicate notification requests
                    AddSqlNotificationRequest()
                End If
            End If
            If intPollResult = -1 Then
                'Exception, no objects
                btnCreateQueues.Text = "&Add SqlNotification Objects"
            Else
                btnCreateQueues.Text = "&Drop SqlNotification Objects"
            End If
        Catch ex As Exception
            'PollNotification's error handler fires
        End Try
    End Sub

    '***********************************
    'Save DataSet Changes Event Handlers
    '***********************************

    Private Sub ProductsBindingSource_CurrentItemChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ProductsBindingSource.CurrentItemChanged
        If NorthwindDataSet.HasChanges Then
            btnSaveChanges.Enabled = True
        Else
            btnSaveChanges.Enabled = False
        End If
    End Sub

    Private Sub ProductsDataGridView_CurrentCellChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ProductsDataGridView.CurrentCellChanged
        ProductsBindingSource.EndEdit()
        If NorthwindDataSet.HasChanges Then
            btnSaveChanges.Enabled = True
        Else
            btnSaveChanges.Enabled = False
        End If
    End Sub

    Private Sub btnSaveChanges_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSaveChanges.Click
        Try
            Me.Cursor = Cursors.WaitCursor
            With NorthwindDataSet
                If .HasChanges Then
                    If ProductsTableAdapter.Update(NorthwindDataSet) > 0 Then
                        .AcceptChanges()
                        'Update the current file
                        NorthwindDataSet.WriteXml(strProductsFile, XmlWriteMode.DiffGram)
                    Else
                        MsgBox("Error updating Products table.", MsgBoxStyle.Information, "Update Error")
                        .RejectChanges()
                    End If
                    btnSaveChanges.Enabled = False
                End If
            End With
        Catch exc As Exception
            Me.Cursor = Cursors.Default
            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Base Table Update Operation Failed")
        Finally
            Me.Cursor = Cursors.Default
        End Try
    End Sub

    '*******************************************
    'SqlDependency Query Notification Procedures
    '*******************************************

    Private Sub AddOrRemoveNotification(ByVal blnAdd As Boolean)
        If depProds IsNot Nothing Then
            'Remove previous handler
            RemoveHandler depProds.OnChange, AddressOf SqlDependency_OnChange
        End If
        If cmdProds IsNot Nothing Then
            'Remove previous notification
            cmdProds.Notification = Nothing
        End If
        If blnAdd Then
            'Create the notification
            Try
                If cnNwind Is Nothing Then
                    cnNwind = New SqlConnection(My.Settings.NorthwindConnection)
                End If
                cmdProds = New SqlCommand(strSQL, cnNwind)

                'Add an SqlDependency for the connection
                depProds = New SqlDependency(cmdProds)
                'Revised 4/8/2006 for SQL Server 2005 SP-1
                depProds.Start(My.Settings.NorthwindConnection)

                'Uncomment below to test timeout
                'cmdProds.Notification.Timeout = 15
                AddHandler depProds.OnChange, New OnChangeEventHandler(AddressOf SqlDependency_OnChange)

                'Execute the command to establish the notification
                cnNwind.Open()
                Dim sdrDep As SqlDataReader = cmdProds.ExecuteReader
                sdrDep.Close()
                cnNwind.Close()

                'Options was Service
                Dim strService As String = "Options: " + cmdProds.Notification.Options
                Dim strTimeout As String = "Timeout: " + cmdProds.Notification.Timeout.ToString + " secs."
                'UserData was ID; UserData isn't available for query notifications; following throws an exception
                'Revised 4/8/2006 for SQL Server 2005 SP-1
                'If cmdProds.Notification.IsUserDataAvailable Then
                If cmdProds.Notification.UserData IsNot Nothing Then
                    Dim strID As String = cmdProds.Notification.UserData
                    'Reformat the GUID pair
                    strID = Replace(strID, ";", ";" + vbCrLf)
                    'Revised 4/8/2006 for SQL Server 2005 SP-1
                    strDepMsg = strService + vbCrLf + strTimeout + vbCrLf + vbCrLf + "UserData: " + _
                        vbCrLf + strID + vbCrLf + vbCrLf
                Else
                    strDepMsg = strService + vbCrLf + strTimeout + vbCrLf + vbCrLf
                End If
            Catch exc As Exception
                MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Create Dependency Operation Failed")
            End Try
        End If
    End Sub

    Private Sub SqlDependency_OnChange(ByVal sender As Object, ByVal args As SqlNotificationEventArgs)
        'Open a message box with query notification details
        'This event-handler runs on its own thread
        Dim blnDataChanged As Boolean
        Dim strMsg As String = Nothing
        strMsg += "Notification query: " + strSQL + vbCrLf
        strMsg += strDepMsg
        strMsg += "Type: " + args.Type.ToString + vbCrLf
        strMsg += "Source: "
        Select Case args.Source
            Case SqlNotificationSource.Data
                blnDataChanged = True
                strMsg += "Data has changed."
            Case SqlNotificationSource.Database
                strMsg += "Database state has changed."
            Case SqlNotificationSource.Environment
                strMsg += "Incompatible run-time environment."
            Case SqlNotificationSource.Execution
                strMsg += "Runtime error occurred."
            Case SqlNotificationSource.Object
                strMsg += "Products table was modified."
            Case SqlNotificationSource.Statement
                strMsg += "Invalid notification query."
            Case SqlNotificationSource.System
                strMsg += "System error."
            Case SqlNotificationSource.Timeout
                strMsg += "Subscription timeout expired."
            Case Else
                strMsg += "Unknown source."
        End Select
        strMsg += vbCrLf + "Info: "
        Select Case args.Info
            Case SqlNotificationInfo.Alter
                strMsg += "Products table was altered."
            Case SqlNotificationInfo.Delete
                strMsg += "Row(s) were deleted."
            Case SqlNotificationInfo.Drop
                strMsg += "Products table was dropped."
            Case SqlNotificationInfo.Error
                strMsg += "Internal server error."
            Case SqlNotificationInfo.Insert
                strMsg += "Row(s) were inserted."
            Case SqlNotificationInfo.Invalid
                strMsg += "Invalid notification query."
            Case SqlNotificationInfo.Isolation
                strMsg += "Invalid transaction isolation mode."
            Case SqlNotificationInfo.Options
                strMsg += "Inappropriate SET options."
            Case SqlNotificationInfo.Query
                strMsg += "Invalid notification SELECT statement."
            Case SqlNotificationInfo.Restart
                strMsg += "Server has been restarted."
            Case SqlNotificationInfo.Truncate
                strMsg += "Products table was truncated."
            Case SqlNotificationInfo.Update
                strMsg += "Row(s) were updated."
            Case Else
                strMsg += "Unknown problem."
        End Select
        strMsg += vbCrLf + "Connection State: " + cnNwind.State.ToString
        If blnDataChanged Then
            strMsg += vbCrLf + vbCrLf + "Click Refresh Data to update the Products lookup table."
        End If
        MsgBox(strMsg, MsgBoxStyle.Information, "Query Notification Received")
        If blnDataChanged Then
            'Recreate the notification
            AddOrRemoveNotification(True)
        End If
    End Sub

    '***********************************************************
    'Enable and Select SqlDependency Notification Event Handlers
    '***********************************************************

    Private Sub chkEnableNotifications_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkEnableNotifications.CheckedChanged
        'Enable or disable notifications
        If chkEnableNotifications.Checked Then
            btnSubscriptions.Enabled = True
            gbQuery.Enabled = True
            AddOrRemoveNotification(True)
        Else
            gbQuery.Enabled = False
            btnSubscriptions.Enabled = False
            AddOrRemoveNotification(False)
        End If
    End Sub

    Private Sub optAllProds_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optAllProds.CheckedChanged
        'Default query
        If chkEnableNotifications.Checked And optAllProds.Checked Then
            strSQL = "SELECT UnitsInStock, UnitsOnOrder, ReorderLevel FROM dbo.Products"
            txtHelp.Text = "All products query: " + strSQL + vbCrLf
            AddOrRemoveNotification(True)
        End If
    End Sub

    Private Sub optReorder_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optReorder.CheckedChanged
        'Detects reorder required or completed
        If chkEnableNotifications.Checked And optReorder.Checked Then
            strSQL = "SELECT UnitsInStock, UnitsOnOrder, ReorderLevel FROM dbo.Products "
            strSQL += "WHERE Discontinued = 0 AND UnitsInStock + UnitsOnOrder - ReorderLevel <= 0"
            txtHelp.Text = "Reorder query: " + strSQL + vbCrLf
            AddOrRemoveNotification(True)
        End If
    End Sub

    Private Sub optInvalid_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles optInvalid.CheckedChanged
        If chkEnableNotifications.Checked And optInvalid.Checked Then
            strSQL = "SELECT TOP 10 UnitsInStock, UnitsOnOrder, ReorderLevel FROM Products"
            txtHelp.Text = "Invalid test query: " + strSQL + vbCrLf
            AddOrRemoveNotification(True)
        End If
    End Sub

    '*******************************************
    'Current subscriptions iteration message box
    '*******************************************

    Private Sub btnSubscriptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSubscriptions.Click
        ListSubscriptions(False)
    End Sub

    Private Function ListSubscriptions(ByVal blnTestSubs As Boolean) As Boolean
        'Returns True if blnTestSubs is True and an SqlNotificationRequest exists 
        If cnNwind Is Nothing Then
            cnNwind = New SqlConnection(My.Settings.NorthwindConnection)
        End If
        Dim cmdSubs As New SqlCommand("SELECT * FROM sys.dm_qn_subscriptions", cnNwind)
        Try
            cnNwind.Open()
            Dim rdrSubs As SqlDataReader = cmdSubs.ExecuteReader
            Dim intCtr As Integer
            Dim strMsg As String = Nothing
            Dim alSubs As New ArrayList
            With rdrSubs
                While .Read
                    If blnTestSubs Then
                        If .GetInt32(5) = Int32.MaxValue Then
                            btnCreateQueues.Text = "&Drop SqlNotification Objects"
                            Return True
                        End If
                    End If
                    If intCtr < 4 Then
                        strMsg += "Notification Type: "
                        If .GetInt32(5) = Int32.MaxValue Then
                            strMsg += "SqlNotificationRequest" + vbCrLf
                        Else
                            strMsg += "SqlDependency" + vbCrLf
                        End If
                        strMsg += "Subscription ID: " + .GetValue(0).ToString + vbCrLf
                        strMsg += "Database ID: " + .GetValue(1).ToString + vbCrLf
                        strMsg += "Object ID: " + .GetValue(2).ToString + vbCrLf
                        strMsg += "User ID: " + .GetValue(3).ToString + vbCrLf
                        strMsg += "Created: " + .GetValue(4).ToString + vbCrLf
                        strMsg += "Timeout, seconds: " + .GetValue(5).ToString + vbCrLf
                        strMsg += "Status ID: " + .GetValue(6).ToString + vbCrLf + vbCrLf
                    End If
                    alSubs.Add(.GetValue(0))
                    intCtr += 1
                End While
                If blnTestSubs Then
                    'There are no subscriptions
                    btnCreateQueues.Text = "&Add SqlNotification Objects"
                    Return False
                End If
            End With
            If intCtr > 4 Then
                strMsg += (intCtr - 4).ToString + " of " + intCtr.ToString + " subscriptions aren't listed." + vbCrLf + vbCrLf
            End If
            strMsg += "Do you want to delete the subscription(s)? (Requires sysadmin role.)"
            rdrSubs.Close()
            If intCtr > 0 Then
                If MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, "Query Notifications: Current Subscriptions") = MsgBoxResult.Yes Then
                    Dim intSub As Integer
                    Dim intDelSubs As Integer
                    'Alternatively, set CommandText to KILL QUERY NOTIFICATION SUBSCRIPTION ALL"
                    For intSub = 0 To intCtr - 1
                        Try
                            cmdSubs.CommandText = "KILL QUERY NOTIFICATION SUBSCRIPTION " + alSubs(intSub).ToString
                            intDelSubs += cmdSubs.ExecuteNonQuery()
                        Catch exc As Exception
                            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Can't Delete Current Subscription")
                        End Try
                    Next
                    If intSub = intCtr Then
                        'All subscriptions removed
                        btnSubscriptions.Enabled = False
                        chkEnableNotifications.Checked = False
                    End If
                End If
            Else
                strMsg = "There are no current query notification subscriptions."
                MsgBox(strMsg, MsgBoxStyle.Information, "Query Notifications: Current Subscriptions")
            End If
            cnNwind.Close()
        Catch exc As Exception
            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Can't Connect to Northwind Database")
        Finally
            cnNwind.Close()
            cmdSubs.Dispose()
        End Try
    End Function

    '*********************************
    'SqlNotificationRequest Procedures
    '*********************************

    Private Sub btnCreateQueues_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCreateQueues.Click
        CreateQueue(False)
    End Sub

    Private Function CreateQueue(ByVal blnSilent As Boolean) As Boolean
        'Create a QUEUE
        Dim strQueueSQL As String = "CREATE QUEUE ProductsQnQueue; "
        'Create a SERVICE with a PostQueryNotification MESSAGE TYPE
        strQueueSQL += "CREATE SERVICE ProductsQnService ON QUEUE ProductsQnQueue" + _
         "([http://schemas.microsoft.com/SQL/Notifications/PostQueryNotification]); "
        'Add an optional ROUTE to the local SQL Server instance
        strQueueSQL += "CREATE ROUTE ProductsQnRoute WITH SERVICE_NAME = 'ProductsQnService', ADDRESS = 'LOCAL'; "
        If cnNwind Is Nothing Then
            cnNwind = New SqlConnection(My.Settings.NorthwindConnection)
        End If
        Dim cmdQueues As New SqlCommand(strQueueSQL, cnNwind)
        Dim strMsg As String
        Try
            cnNwind.Open()
            cmdQueues.ExecuteNonQuery()
            cnNwind.Close()
            'Add an SqlNotificationRequest
            AddSqlNotificationRequest()
            btnSubscriptions.Enabled = True
            strMsg = "Created ProductsQnQueue, ProductsQnService, ProductsQnRoute, " + _
             "and SqlNotificationRequest for All Products Updates query." + vbCrLf + vbCrLf
            strMsg += "Click QN Subscriptions to verify added subscription."
            btnCreateQueues.Text = "&Drop SqlNotification Objects"
            If blnSilent Then
                Return True
            Else
                MsgBox(strMsg, MsgBoxStyle.Information, "Created Database Objects and SqlNotificationRequest")
            End If
        Catch exc As Exception
            cnNwind.Close()
            If blnSilent Then
                Return False
            End If
            If exc.Message.Contains("There is already an object") Then
                strMsg = exc.Message + vbCrLf + vbCrLf + "Do you want to drop the queue, service, route, and all subscriptions?"
                If MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, "Products Queue and Service Exist") = MsgBoxResult.Yes Then
                    Try
                        If cnNwind.State = ConnectionState.Closed Then
                            cnNwind.Open()
                        End If
                        Dim strDropSQL As String = Nothing
                        strDropSQL += "DROP SERVICE ProductsQnService; "
                        strDropSQL += "DROP ROUTE ProductsQnRoute; "
                        strDropSQL += "DROP QUEUE dbo.ProductsQnQueue; "
                        strDropSQL += "KILL QUERY NOTIFICATION SUBSCRIPTION ALL; "
                        cmdQueues.CommandText = strDropSQL
                        cmdQueues.ExecuteNonQuery()
                        cnNwind.Close()
                        btnCreateQueues.Text = "&Add SqlNotification Objects"
                    Catch ex As Exception
                        MsgBox(ex.Message, MsgBoxStyle.Exclamation, "Error Dropping Queue and Service")
                    Finally
                        cnNwind.Close()
                    End Try
                ElseIf Not ListSubscriptions(True) Then
                    'There are no subscriptions
                    strMsg = "Do you want to add a SqlNotificationRequest subscription?"
                    If MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, "The ProductsQnQueue Has No Subscriptions") = MsgBoxResult.Yes Then
                        AddSqlNotificationRequest()
                        btnSubscriptions.Enabled = True
                    End If
                End If
            Else
                MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Error Adding Products Queue")
            End If
        Finally
            cmdQueues.Dispose()
            cnNwind.Close()
        End Try
    End Function

    Private Sub AddSqlNotificationRequest()
        If cnNwind Is Nothing Then
            cnNwind = New SqlConnection(My.Settings.NorthwindConnection)
        End If
        Dim strRequestSQL As String = "SELECT UnitsInStock, UnitsOnOrder, " + _
         "ReorderLevel FROM dbo.Products"
        cmdRequest = New SqlCommand(strRequestSQL, cnNwind)
        'Create an SqlNotification request with the maximum timeout (Int32.MaxValue)
        'Change Int32.MaxValue to 15 to test Timeout

        'Revised 4/8/2006 for SQL Server 2005 SP-1
        'snrProds = New SqlNotificationRequest(Guid.NewGuid().ToString, _
        '  "ProductsQnService", Int32.MaxValue)
        snrProds = New SqlNotificationRequest()
        With snrProds
            .UserData = Guid.NewGuid().ToString()
            .Options = "service=ProductsQnService;local database=Northwind"
            .Timeout = Int32.MaxValue
        End With
        'The current version of VS2005 Online Help's "Using SqlNotificationRequest and 
        'Detecting Notifications" topic has an incorrect value for the .Options property

        'Attach the notification to the command
        cmdRequest.Notification = snrProds
        Try
            'Register 
            cnNwind.Open()
            Dim rdrProds As SqlDataReader = cmdRequest.ExecuteReader
            rdrProds.Close()
            cnNwind.Close()
        Catch exc As Exception
            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "Error Creating SqlNotificationRequest")
        Finally
            cnNwind.Close()
        End Try
    End Sub

    Private Sub btnPollNotifications_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPollNotifications.Click
        PollNotifications(False)
    End Sub

    Private Function PollNotifications(ByVal blnSilent As Boolean) As Integer
        'Poll the ProductsQnQueue for changes
        Dim intRetValue As Integer
        Dim strPollSQL As String = "RECEIVE CAST(message_body AS XML) AS " + _
         "ProductsQnMessage FROM ProductsQnQueue; "
        Dim strProdsMessage As String = Nothing
        Dim strMsg As String = Nothing
        If cnNwind Is Nothing Then
            cnNwind = New SqlConnection(My.Settings.NorthwindConnection)
        End If
        Dim cmdPoll As New SqlCommand(strPollSQL, cnNwind)
        Try
            cnNwind.Open()
            Dim rdrPoll As SqlDataReader = cmdPoll.ExecuteReader
            If rdrPoll.HasRows Then
                'Queue has message(s)
                While rdrPoll.Read()
                    strProdsMessage += "Message: " + rdrPoll(0).ToString + vbCrLf + vbCrLf
                End While
                strMsg = "ProductsQnQueue has changes. " + _
                 "Click Refresh Data to update the Products lookup table." + _
                 vbCrLf + vbCrLf + Replace(strProdsMessage, "><", ">" + vbCrLf + "<")
                cnNwind.Close()
                If Not ListSubscriptions(True) Then
                    'Don't add duplicate notification requests
                    AddSqlNotificationRequest()
                End If
                intRetValue = 1
                MsgBox(strMsg, MsgBoxStyle.Information, "Polling ProductsQnQueue")
            Else
                intRetValue = 0
                strMsg = "ProductsQnQueue is empty, or notification has expired or was deleted."
                If Not blnSilent Then
                    MsgBox(strMsg, MsgBoxStyle.Information, "Polling ProductsQnQueue")
                End If
            End If
            cnNwind.Close()
            Return intRetValue
        Catch exc As Exception
            If Not blnSilent Then
                strMsg = exc.Message
                If exc.Message.Contains("Invalid object") Then
                    strMsg += vbCrLf + vbCrLf + "Click Add SqlNotification Objects to create the queue and service."
                End If
                MsgBox(strMsg, MsgBoxStyle.Exclamation, "Error Polling ProductsQnQueue")
            End If
            'btnCreateQueues.Text = "&Add SqlNotification Objects"
            Return -1 'Error
        Finally
            cnNwind.Close()
            cmdPoll.Dispose()
        End Try
    End Function

    '************************
    'Refresh and save DataSet
    '************************

    Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click
        RefreshProducts(True, True)
    End Sub

    Private Sub RefreshProducts(ByVal blnTest As Boolean, ByVal blnSendDbMail As Boolean)
        'Refill the TableAdapter and save current data
        'If blnTest = True, check for low inventory and mark DataGridView cells
        'Change the following to a valid email address to receive messages
        'Dim strToEmail As String = "someone@mail.somewhere.com"
        Dim strToEmail As String = "oakleaf@sbcglobal.net"
        Dim alReorders As New ArrayList
        Dim alMsgBodies As New ArrayList
        Dim alMessages As New ArrayList
        Me.Cursor = Cursors.WaitCursor
        Try
            ProductsTableAdapter.Fill(NorthwindDataSet.Products)
            NorthwindDataSet.WriteXml(strProductsFile, XmlWriteMode.DiffGram)
            btnSaveChanges.Enabled = False
            If blnTest Then
                'Test for low inventory items
                Dim strMsg As String = Nothing
                Dim strBody As String = Nothing
                Dim strSubject As String = Nothing
                With ProductsBindingSource
                    .MoveFirst()
                    Dim intRow As Integer
                    Dim intFirstRow As Integer = Int32.MaxValue
                    Dim drvItem As DataRowView
                    Dim drSupplier As DataRow()
                    ProductsDataGridView.SuspendLayout()
                    alReorders.Clear()
                    alMsgBodies.Clear()
                    alMessages.Clear()
                    For intRow = 0 To .Count - 1
                        drvItem = CType(.Item(intRow), DataRowView)
                        With drvItem
                            If .Item(6) + .Item(7) - .Item(8) <= 0 Then
                                strMsg += "ProductID " + .Item(0).ToString + " '" + .Item(1).ToString + _
                                 "' requires reorder of " + .Item(8).ToString + " units "

                                Dim strQuan As String = Nothing
                                If .Item(8).ToString = "0" Then
                                    'Default reorder quantity
                                    strQuan = "10"
                                Else
                                    strQuan = .Item(8).ToString
                                End If

                                'Simulate an OrderID
                                Dim strOrderID As String = Now.Year.ToString + Now.Month.ToString + _
                                 Now.Day.ToString + intRow.ToString

                                'Create the DbMail body text
                                strBody = "Please enter our order number " + _
                                strOrderID + " for " + strQuan + " each " + _
                                Replace(.Item(1).ToString, "'", "") + " (" + .Item(4).ToString + _
                                ") and ship immediately by Speedy Express. Northwind standard terms apply." + _
                                 vbCrLf + vbCrLf + "Please acknowledge this order immediately. " + _
                                "Confirmation will follow by postal mail."

                                'Get the supplier data
                                drSupplier = NorthwindDataSet.Suppliers.Select("SupplierID = " + _
                                 drvItem.Item(2).ToString)
                                If drSupplier.Length = 1 Then
                                    With drSupplier(0)
                                        strMsg += "from SupplierID " + .Item(0).ToString + _
                                         " '" + .Item(1).ToString + "'." + vbCrLf + vbCrLf

                                        'Add to the body text
                                        strBody = "To:  " + Replace(.Item(1).ToString, "'", "") + vbCrLf + _
                                         "Att: " + Replace(.Item(2).ToString, "'", "") + ", " + _
                                         .Item(3).ToString + vbCrLf + vbCrLf + strBody + _
                                         vbCrLf + vbCrLf + "Jose Santiago, Purchasing Agent" + _
                                         vbCrLf + "joses@northwind.com"

                                        'Create the reorder DbMail command text
                                        'An EXECUTE command is used here for clarity (and brevity)
                                        'The SendReordersByDbMail procedure adds parameters to the command
                                        strSubject = "Northwind Traders Purchase Order for " + _
                                           Replace(drvItem.Item(1).ToString, "'", "")
                                        Dim strReorder As String = "EXECUTE dbo.sp_send_dbmail " + _
                                         "@profile_name = 'NorthwindSuppliers', " + _
                                         "@recipients = '" + strToEmail + "', " + _
                                         "@subject = '" + strSubject + "', " + _
                                         "@body = '" + strBody + "', " + _
                                         "@body_format = 'TEXT', " + _
                                         "@importance = 'HIGH'"

                                        'Add the command text to the ArrayList
                                        alReorders.Add(strReorder)
                                        alMsgBodies.Add(strBody)

                                        'Add the message details to the ArrayList
                                        Dim objReorder As New clsReorder
                                        With objReorder
                                            .ToEmail = strToEmail
                                            .Subject = strSubject
                                            .Body = strBody
                                        End With
                                        alMessages.Add(objReorder)
                                    End With
                                End If
                                With ProductsDataGridView
                                    'Change to yellow background
                                    .Rows(intRow).Cells(1).Style.BackColor = Color.Yellow
                                    .Rows(intRow).Cells(6).Style.BackColor = Color.Yellow
                                End With
                                If intFirstRow = Int32.MaxValue Then
                                    intFirstRow = intRow
                                End If
                            Else
                                With ProductsDataGridView
                                    'White background
                                    .Rows(intRow).Cells(1).Style.BackColor = Color.White
                                    .Rows(intRow).Cells(6).Style.BackColor = Color.White
                                End With
                            End If
                        End With
                        .MoveNext()
                    Next
                    If intFirstRow = Int32.MaxValue Then
                        .MoveFirst()
                    Else
                        .Position = intFirstRow
                    End If
                    ProductsDataGridView.ResumeLayout()

                    If alReorders.Count > 0 And chkSendMessages.Checked Then
                        If chkSendDbMail.Checked Then
                            'Process automated reorders with DbMail
                            SendReordersByDbMail(alReorders, alMessages, strMsg)
                        Else
                            'Process automated reorders with SmptClient
                            SendReordersBySMTP(alMessages, strMsg)
                        End If
                    End If
                End With
            End If
        Catch exc As Exception
            Me.Cursor = Cursors.Default
            MsgBox(exc.Message, MsgBoxStyle.Exclamation, "DataSet Refresh Operation Failed")
        Finally
            Me.Cursor = Cursors.Default
            cnNwind.Close()
        End Try
    End Sub

    Private Sub SendReordersByDbMail(ByVal alReorders As ArrayList, ByVal alMessages As ArrayList, ByVal strMsg As String)
        'If set up, use DbMail to send messages with test for duplicates
        Dim rdrTest As SqlDataReader = Nothing
        Dim cmdReorder As SqlCommand = Nothing
        Dim strToEmail As String = Nothing
        Dim strSubject As String = Nothing
        Dim strBody As String = Nothing
        Try
            Dim strMailSQL As String = "SELECT COUNT(*) FROM dbo.sysmail_mailitems"
            Dim cmdTestMail As New SqlCommand(strMailSQL, cnNwind)
            cnNwind.Open()
            'Following throws an exception if DbMail isn't installed
            Dim intMailCount As Integer = CType(cmdTestMail.ExecuteScalar(), Integer)
            cnNwind.Close()
            strMsg += "Send automated reorders by DbMail?"
            If MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, _
             "Products with Units in Stock below Reorder Level") = MsgBoxResult.Yes Then
                Dim intReorder As Integer
                Dim intSent As Integer
                Dim bytStatus As Byte
                Dim datSent As DateTime
                Dim strDupSubject As String = Nothing
                Dim strDupBody As String = Nothing
                cnNwind.Open()
                For intReorder = 0 To alReorders.Count - 1
                    Dim blnUseParams As Boolean = True
                    If blnUseParams Then
                        'Create the parameterized command
                        strToEmail = alMessages(intReorder).ToEmail
                        strSubject = alMessages(intReorder).Subject
                        strBody = alMessages(intReorder).Body
                        cmdReorder = New SqlCommand("dbo.sp_send_dbmail", cnNwind)
                        With cmdReorder
                            .CommandType = CommandType.StoredProcedure
                            .Parameters.AddWithValue("@profile_name", "NorthwindSuppliers")
                            .Parameters.AddWithValue("@recipients", strToEmail)
                            .Parameters.AddWithValue("@subject", strSubject)
                            .Parameters.AddWithValue("@body", strBody)
                            .Parameters.AddWithValue("@body_format", "TEXT")
                            .Parameters.AddWithValue("@importance", "HIGH")
                            Dim prmRetVal As New SqlParameter("RETURN_VALUE", SqlDbType.Int)
                            prmRetVal.Direction = ParameterDirection.ReturnValue
                            .Parameters.Add(prmRetVal)
                        End With
                    Else
                        'Create the EXECUTE command
                        cmdReorder = New SqlCommand(alReorders(intReorder).ToString, cnNwind)
                    End If

                    Dim blnDateIsNull As Boolean
                    'Test for duplicate messages (dependent on same purchase order number)
                    Dim strTestSQL As String = "SELECT sent_status, sent_date, " + _
                    "subject, body FROM dbo.sysmail_mailitems " + _
                    "WHERE body = '" + alMessages(intReorder).Body + "'"
                    Dim cmdTest As New SqlCommand(strTestSQL, cnNwind)
                    Dim blnHasRows As Boolean
                    rdrTest = cmdTest.ExecuteReader
                    With rdrTest
                        If .HasRows Then
                            blnHasRows = True
                            'Get the last duplicate
                            While .Read
                                bytStatus = .GetByte(0)
                                If .Item(1) Is DBNull.Value Then
                                    blnDateIsNull = True
                                Else
                                    datSent = .GetDateTime(1)
                                End If
                                strDupSubject = .GetString(2)
                                strDupBody = .GetString(3)
                            End While
                        End If
                        .Close()
                    End With
                    If blnHasRows Then
                        Dim strStatus As String = "A duplicate of this message "
                        If bytStatus = 0 Then
                            strStatus += "was pending"
                        ElseIf bytStatus = 1 Then
                            strStatus += "was sent to the e-mail server"
                        Else
                            strStatus += "failed delivery to the e-mail server"
                        End If
                        If Not blnDateIsNull Then
                            strStatus += " on " + datSent.ToShortDateString
                            strStatus += " at " + datSent.ToShortTimeString
                        End If
                        strStatus += "." + vbCrLf + vbCrLf
                        strStatus += "Subject: " + strDupSubject + vbCrLf + vbCrLf
                        strStatus += strDupBody + vbCrLf + vbCrLf
                        strStatus += "Do you want to resend this reorder?"
                        If MsgBox(strStatus, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, "Duplicate Reorder Message Found") = MsgBoxResult.Yes Then
                            intSent += cmdReorder.ExecuteNonQuery
                        End If
                    Else
                        intSent += cmdReorder.ExecuteNonQuery
                    End If
                    If blnUseParams Then
                        'Returns 1 if an error
                        Dim intRetVal As Integer = cmdReorder.Parameters("RETURN_VALUE").Value
                    End If
                Next
                cnNwind.Close()
                If intSent < 0 Then
                    strSubject = alMessages(alMessages.Count - 1).Subject
                    strBody = alMessages(alMessages.Count - 1).Body
                    strMsg = (-intSent).ToString + " messages sent by DbMail. Last message:" + vbCrLf + vbCrLf
                    strMsg += "Subject: " + strSubject + vbCrLf + vbCrLf + strBody
                Else
                    strMsg = "No messages were sent by DbMail"
                End If
                MsgBox(strMsg, MsgBoxStyle.Information, "Automated Reorder Message Report - DbMail")
            Else
                strSubject = alMessages(alMessages.Count - 1).Subject
                strBody = alMessages(alMessages.Count - 1).Body
                strMsg = "No messages were sent by DbMail. Last message would have been:" + vbCrLf + vbCrLf
                strMsg += "Subject: " + strSubject + vbCrLf + vbCrLf + strBody
                MsgBox(strMsg, MsgBoxStyle.Information, "Automated Reorder Messages Canceled")
            End If
        Catch excMail As Exception
            If excMail.Message.Contains("Invalid object name") Then
                'DbMail isn't installed
                cnNwind.Close()
                strMsg += "(Can't mail reorder messages because DbMail isn't installed.)"
                strMsg += vbCrLf + vbCrLf + "Display last message details?"
                If MsgBox(strMsg, MsgBoxStyle.Exclamation Or MsgBoxStyle.YesNo, _
                "Products with Units in Stock below Reorder Level") = MsgBoxResult.Yes Then
                    strSubject = alMessages(alMessages.Count - 1).Subject
                    strBody = alMessages(alMessages.Count - 1).Body
                    strMsg = "No messages sent by DbMail. Last message would be:" + vbCrLf + vbCrLf
                    strMsg += "Subject: " + strSubject + vbCrLf + vbCrLf + strBody
                    MsgBox(strMsg, MsgBoxStyle.Information, "Emulated Automated Reorder Message Report")
                End If
            Else
                MsgBox(excMail.Message, MsgBoxStyle.Exclamation, "DbMail Error")
            End If
        Finally
            If rdrTest IsNot Nothing Then
                rdrTest.Close()
            End If
            If cmdReorder IsNot Nothing Then
                cmdReorder.Dispose()
            End If
            cnNwind.Close()
        End Try
    End Sub

    Private Sub SendReordersBySMTP(ByVal alMessages As ArrayList, ByVal strMsg As String)
        'Emulate DbMail with the SmtpClient object

        'Replace these values with those for your e-mail provider
        Dim strHost As String = "smtp.mail.somewhere.com"
        Dim strFromEmail As String = "someone@mail.somewhere.com"
        Dim strPassword As String = "somewhere"

        Dim strToEmail As String = Nothing
        Dim strSubject As String = Nothing
        Dim strBody As String = Nothing

        strMsg += "Send automated reorders to SMTP host with SmtpClient?"
        If MsgBox(strMsg, MsgBoxStyle.Question Or MsgBoxStyle.YesNo, _
         "Products with Units in Stock below Reorder Level") = MsgBoxResult.Yes Then
            Try
                'Create an SmtpClient
                Dim cliReorder As New SmtpClient(strHost)
                'If host uses Windows authentication, uncomment below
                'cliReorder.UseDefaultCredentials = True
                'Otherwise add mail provider authentication credentials
                Dim crdHost As New NetworkCredential(strFromEmail, strPassword)
                cliReorder.Credentials = crdHost

                Dim intCtr As Integer
                For intCtr = 0 To alMessages.Count - 1
                    'Create the MailMessage
                    strToEmail = alMessages(intCtr).ToEmail
                    strSubject = alMessages(intCtr).Subject
                    strBody = alMessages(intCtr).Body
                    Dim msgReorder As New MailMessage(strFromEmail, strToEmail, strSubject, strBody)
                    'Send the message
                    cliReorder.Send(msgReorder)
                Next
                If intCtr > 0 Then
                    strMsg = intCtr.ToString + " messages sent to SMTP host. Last message:" + vbCrLf + vbCrLf
                    strMsg += "Subject: " + strSubject + vbCrLf + vbCrLf + strBody
                Else
                    strMsg = "No messages were sent to SMTP host."
                End If
                MsgBox(strMsg, MsgBoxStyle.Information, "Automated Reorder Message Report - SMTP")
            Catch excMail As Exception
                MsgBox(excMail.Message, MsgBoxStyle.Exclamation, "SmtpClient Error")
            End Try
        Else
            strSubject = alMessages(alMessages.Count - 1).Subject
            strBody = alMessages(alMessages.Count - 1).Body
            strMsg = "No messages sent to SMTP host. Last message would have been:" + vbCrLf + vbCrLf
            strMsg += "Subject: " + strSubject + vbCrLf + vbCrLf + strBody
            MsgBox(strMsg, MsgBoxStyle.Information, "Emulated Automated Reorder Message Report - SMTP")
        End If
    End Sub

    Private Class clsReorder
        'For SMTP mail
        Public ToEmail As String
        Public Subject As String
        Public Body As String
    End Class

    Private Sub frmNotifications_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        'Save the current data before closing
        NorthwindDataSet.WriteXml(strProductsFile, XmlWriteMode.DiffGram)
        'Revised 4/8/2006 for SQL Server 2005 SP-1
        depProds.Stop(My.Settings.NorthwindConnection)
    End Sub

    Private Sub chkSendMessages_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkSendMessages.CheckedChanged
        If chkSendMessages.Checked Then
            chkSendDbMail.Enabled = True
        Else
            chkSendDbMail.Enabled = False
        End If
    End Sub
End Class
