USE Northwind;
GO

IF EXISTS (SELECT * FROM sys.objects WHERE name = 'ActivationProc') DROP PROCEDURE ActivationProc;    
GO
CREATE PROCEDURE ActivationProc
AS
PRINT 'Starting activation procedure ...'
WHILE (1 = 1)
BEGIN 
   DECLARE @conversation_handle UNIQUEIDENTIFIER,
           @conversation_group_id UNIQUEIDENTIFIER,
           @message_body XML,
           @message_type_name NVARCHAR(128);
   BEGIN TRANSACTION;
   WAITFOR(GET CONVERSATION GROUP @conversation_group_id FROM dbo.NwindTargetQueue),
      TIMEOUT 500;
   IF @conversation_group_id IS NULL
   BEGIN
      ROLLBACK TRANSACTION;
      BREAK;
   END;
   WHILE (1 = 1)
   BEGIN
      RECEIVE TOP(1)
         @conversation_handle = conversation_handle,
         @message_type_name = message_type_name,
         @message_body = CAST(message_body AS XML)
      FROM dbo.NwindTargetQueue
      WHERE conversation_group_id = @conversation_group_id;
      IF @@ROWCOUNT = 0 OR @@ERROR <> 0 BREAK;
      IF @message_type_name = 'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
         OR @message_type_name = 'http://schemas.microsoft.com/SQL/ServiceBroker/Error'
         BEGIN
            END CONVERSATION @conversation_handle;
         END;
      ELSE
         PRINT 'Conversation Group Id: ' +  CONVERT(varchar(100), @conversation_group_id) +
         '; Conversation Handle: ' +  CONVERT(varchar(100), @conversation_handle) +
         '; Message Type Name: '  +  CONVERT(varchar(100), @message_type_name) +
         '; Message Body: ' +  +  CONVERT(varchar(100), @message_body);
   END;
   COMMIT TRANSACTION;
END;
GO

DROP Service NwindTargetService;
GO

DROP QUEUE dbo.NwindTargetQueue;
GO

CREATE QUEUE dbo.NwindTargetQueue 
   WITH STATUS = ON, RETENTION = OFF,
   ACTIVATION( STATUS = ON,
   PROCEDURE_NAME = ActivationProc,
   MAX_QUEUE_READERS = 2, 
   EXECUTE AS SELF );
GO

CREATE SERVICE NwindTargetService ON QUEUE dbo.NwindTargetQueue (NwindContract);
GO

USE master;
GO
